<?php

/**
 * This file contains QUI\ERP\Products\Search\Utils
 */

namespace QUI\ERP\Products\Utils;

use QUI;

/**
 * Class Search
 */
class Search
{
    /**
     * List of default frontend category fields
     *
     * @var null
     */
    protected static $defaultFrontendFields = null;

    /**
     * List of default free text fields
     *
     * @var null
     */
    protected static $defaultFreeTextFields = null;

    /**
     * @return array
     */
    public static function getSearchParameterFromRequest()
    {
        $search = QUI::getRequest()->get('search');
        $fields = QUI::getRequest()->get('f');
        $tags   = QUI::getRequest()->get('t');
        $sortBy = QUI::getRequest()->get('sortBy');
        $sortOn = QUI::getRequest()->get('sortOn');

        if (isset($_REQUEST['search'])) {
            $search = $_REQUEST['search'];
        }

        $categories = QUI::getRequest()->get('c');

        $Site           = QUI::getRewrite()->getSite();
        $defaultSorting = $Site->getAttribute('quiqqer.products.settings.defaultSorting');

        if (empty($defaultSorting)) {
            $Package        = QUI::getPackage('quiqqer/products');
            $defaultSorting = $Package->getConfig()->get('products', 'defaultSortField');
        }

        if ($categories) {
            $categories = \explode(',', $categories);
        }

        if (!\is_array($categories)) {
            $categories = [];
        }

        // look for default site settings
        if (!$sortOn && !empty($defaultSorting)) {
            $sorting = \explode(' ', $defaultSorting);
            $sortOn  = $sorting[0];

            if (\strpos($sortOn, 'S') === 0) {
                $sortOn = \mb_substr($sortOn, 1);
            }
        }

        if (!$sortBy && !empty($defaultSorting)) {
            $sorting = \explode(' ', $defaultSorting);

            if (isset($sorting[1])) {
                switch ($sorting[1]) {
                    case 'DESC':
                    case 'ASC':
                        $sortBy = $sorting[1];
                }
            }
        }

        $searchParams = \array_filter([
            'freetext' => $search,
            'fields'   => $fields,
            'tags'     => $tags,
            'sortBy'   => $sortBy,
            'sortOn'   => $sortOn,
        ]);

        if (!empty($categories)) {
            $searchParams['categories'] = $categories;
        }

        if (isset($searchParams['fields'])) {
            $searchParams['fields'] = \json_decode($searchParams['fields'], true);

            if (\is_null($searchParams['fields']) && !empty($_REQUEST['f'])) {
                $searchParams['fields'] = \json_decode($_REQUEST['f'], true);
            }

            if (\is_null($searchParams['fields'])) {
                unset($searchParams['fields']);
            }
        }

        if (isset($searchParams['tags'])) {
            $searchParams['tags'] = \explode(',', $searchParams['tags']);
        }

        return $searchParams;
    }

    /**
     * @return mixed|string
     */
    public static function getViewParameterFromRequest()
    {
        if (QUI::getSession()->get('productView')) {
            return QUI::getSession()->get('productView');
        }

        if (QUI::getRequest()->get('v')) {
            return QUI::getRequest()->get('v');
        }

        return '';
    }

    /**
     * Return the default frontend fields
     *
     * @return array
     */
    public static function getDefaultFrontendFields()
    {
        if (self::$defaultFrontendFields !== null) {
            return self::$defaultFrontendFields;
        }

        $Package    = QUI::getPackage('quiqqer/products');
        $defaultIds = $Package->getConfig()->get('search', 'frontend');
        $fields     = [];

        if ($defaultIds) {
            $defaultIds = \explode(',', $defaultIds);

            foreach ($defaultIds as $fieldId) {
                try {
                    $fields[] = QUI\ERP\Products\Handler\Fields::getField($fieldId);
                } catch (QUI\Exception $Exception) {
                }
            }
        }

        self::$defaultFrontendFields = $fields;

        return $fields;
    }


    /**
     * Return the default frontend fields
     *
     * @return array
     */
    public static function getDefaultFrontendFreeTextFields()
    {
        if (self::$defaultFreeTextFields !== null) {
            return self::$defaultFreeTextFields;
        }

        $Package    = QUI::getPackage('quiqqer/products');
        $defaultIds = $Package->getConfig()->get('search', 'freetext');
        $fields     = [];

        if ($defaultIds) {
            $defaultIds = \explode(',', $defaultIds);

            foreach ($defaultIds as $fieldId) {
                try {
                    $fields[] = QUI\ERP\Products\Handler\Fields::getField($fieldId);
                } catch (QUI\Exception $Exception) {
                }
            }
        }

        self::$defaultFreeTextFields = $fields;

        return $fields;
    }
}
